# -*- coding: utf-8 -*-

from __future__ import unicode_literals

__author__ = 'Dominik Janků'
__license__ = 'BSD'

import math
from fractions import Fraction


class InvalidSegmentError(AttributeError):
    def __init__(self, segment):
        self.segment = segment

    def __str__(self):
        return self.segment


class Point(object):
    """
    Třída reprezentující rovinný bod.
    """
    def __init__(self, x, y, name=None, tag=None, vertex=None):
        self.x = x
        self.y = y
        self.tag = tag
        self.name = name
        self.vertex = vertex

    def __eq__(self, other):
        if not isinstance(other, Point):
            return NotImplemented
        return self.x == other.x and self.y == other.y

    def __ne__(self, other):
        return not self.__eq__(other)

    def __lt__(self, other):
        if not isinstance(other, Point):
            return NotImplemented
        return self.y < other.y or (self.y == other.y and self.x < other.x)

    def __gt__(self, other):
        if not isinstance(other, Point):
            return NotImplemented
        return self.y > other.y or (self.y == other.y and self.x > other.x)

    def __le__(self, other):
        return not self.__gt__(other)

    def __ge__(self, other):
        return not self.__lt__(other)

    def __repr__(self):
        return "<Point({0},{1})>".format(self.x, self.y)

    def __str__(self):
        return "[{0}; {1}]".format(self.x, self.y)

    def __hash__(self):
        return (self.x, self.y).__hash__()

    def distance(self, point):
        return math.sqrt((self.x-point.x)**2 + (self.y-point.y)**2)


class Segment(object):
    """
    Třída reprezentující úsečku.
    """
    def __init__(self, a, b, name=None, tag=None, edge=None):
        """
        :type a: Point
        :type b: Point
        """
        if a == b:
            raise AttributeError("a cannot be equal to b")
        self.a = a
        self.b = b
        self.tag = tag
        self.name = name
        # pomocna promenna, muze obsahovat odkaz na hranu, ze ktere je usecka tvorena
        self.edge = edge

    def __str__(self):
        return "({0}, {1})".format(self.a, self.b) if self.name is None else self.name

    def __repr__(self):
        return "<Segment({0}, {1})>".format(repr(self.a), repr(self.b))

    def __hash__(self):
        return (self.a, self.b).__hash__()

    @property
    def upper_point(self):
        """
        Vyšší bod.
        :rtype: Point
        """
        return self.a if self.a < self.b else self.b

    @property
    def lower_point(self):
        """
        Nižší bod.
        :rtype: Point
        """
        return self.a if self.a > self.b else self.b

    @property
    def edges(self):
        """
        Vrati hrany, ze kterych se usecka sklada (pokud je hrana nastavena).
        """
        if self.edge is not None:
            return self.edge, self.edge.twin

    def get_x_by_y(self, y):
        """
        Vrati x-pozici usecky, kterou protina primka y

        :type y: float
        :rtype: Fraction
        """
        # horizontalni primka by zpusobila deleni nulou, vratim levy zacatek x
        if self.b.y == self.a.y:
            raise ArithmeticError("Zadne nebo nekonecne mnoho reseni")
        else:
            numerator = Fraction((Fraction(y)-self.a.y) * Fraction(self.b.x - self.a.x))
            denominator = Fraction(self.b.y - self.a.y)
            return Fraction(self.a.x) + Fraction(numerator, denominator)

    def has_intersection(self, other):
        """
        Vrati prusecik s druhou useckou, pokud existuje.

        :param other: Other segment.
        :type other: Segment
        :return: Intersection point or None.
        :rtype: Point or None
        """
        # pro vypocet resim rovnici
        # ax+t(bx-ax)=cx+u(dx-cx)
        # ay+t(by-ay)=cy+u(dy-cy)

        ax, ay, bx, by = self.a.x, self.a.y, self.b.x, self.b.y
        cx, cy, dx, dy = other.a.x, other.a.y, other.b.x, other.b.y
        u1, u2 = bx-ax, by-ay
        v1, v2 = dx-cx, dy-cy

        t1_denominator = Fraction(u1*v2-u2*v1)

        # otestuji usecky na rovnobeznost ...
        if t1_denominator == 0:
            return None

        # kontrole zda-li prusecik lezi na vymezenych useckach (ne primkach)
        t = Fraction(Fraction(v1*(ay-cy)+v2*(cx-ax)), t1_denominator)
        #print "t: ", t
        if not 0 <= t <= 1:
            return None

        u = Fraction(Fraction((ax-cx)*u2+(cy-ay)*u1), Fraction(v1*u2-v2*u1))
        #print "u: ", u
        if not 0 <= u <= 1:
            return None

        return Point(Fraction(t*u1+ax), Fraction(t*u2+ay))
