#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
"""
    Demonstrativní aplikace na překryv map.
"""

from __future__ import unicode_literals

__author__ = 'Dominik Janků'
__license__ = 'BSD'
__version__ = '1.0'

import os
from collections import defaultdict, namedtuple
from math import sqrt
from decimal import *
from Tkinter import *
import tkFileDialog

from blist import sortedset, sortedlist, sorteddict

from dcel import DCEL, Edge, Vertex, Face
from geotools import Segment, Point

# preciznost na 8 desetinnych mist
getcontext().prec = 8


class Cycle(object):
    def __init__(self, index=None, face_labels=set()):
        self.index = index
        self.face_labels = face_labels
        self.leftmost_vertex = None
        self.leftmost_edge = None
        self.component = None

        self.edges = []

    def __repr__(self):
        return "<" + '->'.join(map(lambda x: x.name, self.edges)) + ">"

    def is_inner(self):
        # jedna se o neomezenou obast f0
        if self.index == 0:
            return False
        e = self.leftmost_edge
        p = e.prev.origin.point
        q = e.origin.point
        r = e.next.origin.point
        return (q.x-p.x)*(p.y-r.y)-(r.x-p.x)*(p.y-q.y) < 0

    def is_outer(self):
        return not self.is_inner()


class Component(object):
    def __init__(self):
        self.outer = None
        self.inners = []
        self.face_labels = set()

    def __repr__(self):
        return "<Component({0}| {1})>".format(
            self.outer.index,
            ",".join(map(lambda x: str(x.index), self.inners)))


class MapOverlayAlgorithm(object):
    """
    Trida reprezentujici algoritmus zametaci primky.
    """

    SearchSegClass = namedtuple('SearchSeg', ['a', 'b', 'name'])

    def __init__(self, canvas=None):
        # promenne simulatoru + algoritmu
        self.canvas = canvas

        #jednotlive kroky algoritmu
        self.steps = [
            ('input', 'Načtení map'),
            ('init', 'Sloučení map do jednoho DCEL'),
            ('events', 'Inicializace bodů události'),
            ('sweepline', 'Obsluha události p'),
            ('sweepline-sets', 'zpracování U(p), C(p), L(p)'),
            ('sweepline-split', 'rozdělení hran C(p)'),
            ('sweepline-connect', 'propojení hran a aktualizace T'),
            ('sweepline-findnew', 'hledání nových průsečíků'),
            ('cycles', 'Detekce cyklů'),
            ('components', 'Komponenty souvislosti'),
            ('faces', 'Oblasti'),
            ('finish', 'Dokončený překryv map'),
        ]

        self.step_codes = map(lambda s: s[0], self.steps)
        self.steps_count = len(self.steps)

        # index kroku
        self.step = None
        self.next_step = 'input'
        self.prev_step = None

        self.can_step = True

        # text zobrazovany u daneho kroku
        self.step_info = ''

        # promenne algoritmu
        self.events = sortedset()
        self.event = None

        # pomocna promenna (treba pro dilci kroky jednoho kroku)
        self._index = 0

        # dvojite souvisly seznam prvni mapy
        self.map1 = DCEL()

        # dvojite souvisly seznam druhe mapy
        self.map2 = DCEL()

        # dvojite souvisly seznam vysledne mapy
        self.map = DCEL('mapa')

        # vyhledavani na existenci vrcholu pomoci bodu, odkaz na vrchol tag
        self.points = sortedlist()

        # jednotlive usecky {canvas_item: Segment}
        self.segments = list()

        # zaznamenava usecky vlevo od bodu udalosti p
        self.left_segments = sorteddict()

        # cykly
        self.cycles = []

        # komponenty
        self.components = []

        # prusecik: mnozina usecek
        self.intersections = dict()

        self.Up = defaultdict(sortedset)
        self.Cp = defaultdict(sortedset)
        self.Lp = defaultdict(sortedset)

        # udalost, podle ktere se bude urcovat poradi usecek v T
        self._tree_event = None
        self.tree = sortedlist(key=self.tree_segment_order)

        # aktualni mnoziny
        self._up = None
        self._cp = None
        self._lp = None

        self._ucp_leftmost = None
        self._ucp_rightmost = None

    def info_event(self):
        return "Událost p=[%.1f;%.1f]\n\n" % (self.event.x, self.event.y)

    def info_sets(self):
        return "L(p)={" + ", ".join([x.name for x in self._lp]) + "}\n" + \
               "C(p)={" + ", ".join([x.name for x in self._cp]) + "}\n" + \
               "U(p)={" + ", ".join([x.name for x in self._up]) + "}\n"

    def info_tree(self):
        text = "\nPořadí úseček ve stromě T:\n"
        if len(self.tree):
            text += ", ".join([s.name for s in self.tree]) + '\n'
        else:
            text += 'žádná úsečka\n'
        return text

    def info_components(self):
        text = ''
        for i, g in enumerate(self.components):
            text += 'g{0}: c{1}'.format(i, g.outer.index)
            if len(g.inners):
                text += " | " + ", ".join(map(lambda x: 'c'+str(x.index),
                                           g.inners))
            text += '\n'
        return text

    def tree_segment_order(self, s):
        """
        Urcuje poradi usecek ve strome T.
        :type s: Segment
        """
        p = self._tree_event

        a = min(s.a, s.b)
        b = max(s.a, s.b)

        x = Decimal(str(float(s.get_x_by_y(p.y)))[:-1]) if a.y != b.y else p.x
        try:
            alfa = (b.x-a.x) / sqrt((b.x-a.x)**2 + (b.y-a.y)**2)
        except ZeroDivisionError:
            alfa = 0

        return x, alfa

    def find_new_event(self, s1, s2):
        """
        Pokusi se nalezt novou udalost u dvou usecek.
        :type s1: Segment
        :type s2: Segment
        """
        if s1 is None or s2 is None:
            return None

        # otestuju dve usecky na jejich prusecik
        p = s1.has_intersection(s2)
        if p is None:
            return None

        p = Point(float(str(float(p.x))[:-2]), float(str(float(p.y))[:-2]))

        # nova udalost lezi "nad" bodem stavajici udalosti => nezajima nas
        if p <= self.event:
            return None

        if p not in (s1.a, s1.b):
            self.Cp[p].add(s1)
        if p not in (s2.a, s2.b):
            self.Cp[p].add(s2)

        # ano, zjistena nova udalost
        if p not in self.events:
            self.events.add(p)
            return p
        return None

    def clockwise_connect(self, s, t):
        """
        Nastavi naslednika a predchudce.
        """
        e1 = s.edge if s.edge.origin.point != self.event else s.edge.twin
        e2 = t.edge if t.edge.origin.point == self.event else t.edge.twin
        e1.next = e2
        e2.prev = e1

    def split_cp_segment(self, s, p):
        """
        Zajisti rozdeleni a vytvoreni novych hran + prislusne usecky,
        kdyz je obsluha s z C(p).
        """
        e1 = s.edge if s.edge.origin.point < p else s.edge.twin
        e2 = e1.twin

        # existuje jiz nejaky vrchol se souradnicemi v bode p?
        if p in self.points:
            origin = self.points[self.points.index(p)].vertex
        # pokud ne, tak jej musim vytvorit
        else:
            origin = Vertex(name=s.edge.origin.name+"'", point=p, edge=None)
            p.vertex = origin
            self.map.vertices.append(origin)
            self.points.add(p)

        # vytvorime novou hranu e1'
        e11 = Edge(e1.name+"'", origin=origin, twin=e1, next=e2.next, prev=e2)
        e11.next.prev = e11
        e2.next = e11
        e1.twin = e11
        setattr(e11, 'source_map', e1.source_map)
        setattr(e11, 'face_label', e2.face_label)

        # vytvorime novou hranu e2'
        e22 = Edge(e2.name+"'", origin=origin, twin=e2, next=e1.next, prev=e1)
        e22.next.prev = e22
        e1.next = e22
        e2.twin = e22
        setattr(e22, 'source_map', e2.source_map)
        setattr(e22, 'face_label', e1.face_label)

        if origin.edge is None:
            origin.edge = e11

        self.map.edges.append(e11)
        self.map.edges.append(e22)

        # nastavime vazby mezi hranama a useckou
        old_s = e1.segment
        old_s.a = e1.origin.point
        old_s.b = p
        old_s.edge = e11
        e11.segment = old_s
        e1.segment = old_s

        new_s = Segment(p, e2.origin.point, name='s'+str(len(self.segments)), edge=e2)
        e2.segment = new_s
        e22.segment = new_s
        new_s.edge = e22

        self.segments.append(new_s)

        return new_s

    def do_step(self):
        """
        Provede krok, ktery je definovany jako dalsi, tj. self.next_step.
        """
        self.prev_step = self.step
        self.step = self.next_step
        self.step_info = ''
        handler = 'do_step_'+self.step.replace('-', '_')
        if hasattr(self, handler):
            f = getattr(self, handler)
            return f()

    def do_step_input(self):
        self.next_step = 'init'
        self.step_info = "Vyberte prosím mapy pro výpočet překryvu pomocí menu: \nSoubor -> Vybrat x. mapu..."
        self.step_info += "\n\nPrvní mapa bude zobrazena bílou barvou, druhá žlutou."
        self.step_info += "\n\nPo vybrání obou map budete moct pokračovat dále."

        self.can_step = False

    def do_step_init(self):
        """
        Tento krok zajisti slouceni vrcholu a hran dvou DCEL do jednoho.
        Navic prevede hrany na usecky pro potreby pocitani pruseciku.
        """
        self.next_step = 'events'
        self.step_info = "Dvojitě souvislé seznamy map byly sloučeny do jednoho.\n\n"
        self.can_step = True

        # oznacime vsechny vrcholy jako nepridane
        map(lambda v: setattr(v, 'is_added', False), self.map1.vertices)
        map(lambda v: setattr(v, 'is_added', False), self.map2.vertices)

        for e in self.map1.edges:
            setattr(e, 'source_map', 1)
            setattr(e, 'face_label', e.face.name)
            self.map.edges.append(e)

            v = e.origin
            if not v.is_added:
                self.map.vertices.append(v)
                v.is_added = True
                # pridame bod do vyhledavaciho pole, abychom mohli pri
                # slucovani druhe mapy zjistit, jestli existuje bod se stejnymi souradnicemi
                p = Point(v.point.x, v.point.y, vertex=v)
                self.points.add(p)

        for e in self.map2.edges:
            setattr(e, 'source_map', 2)
            setattr(e, 'face_label', e.face.name)
            self.map.edges.append(e)

            v = e.origin
            if v.point in self.points:
                w = self.points[self.points.index(v.point)].vertex
                e.origin = w
            elif not v.is_added:
                self.map.vertices.append(v)
                v.is_added = True

        # vymazeme jiz nepotrebny atribut o pridani/nepridani vrcholu
        map(lambda v: delattr(v, 'is_added'), self.map.vertices)

        # kazda mapa ma po nacteni ze souboru take aktualizovany seznam usecek
        # ulozeny v mapN.segments, neni proto treba je znovu zjistovat,
        # jednoduse je spojime
        self.map.segments = self.map1.segments + self.map2.segments
        self.segments = self.map.segments

        # nastavime jmena usecek pro lepsi orientaci
        for i in xrange(len(self.segments)):
            self.segments[i].name = "s"+str(i)

        self.step_info += "Počet vrcholů: {0}\nPočet hran: {1}\n".format(
            len(self.map.vertices), len(self.map.edges))

        return self.segments

    def do_step_events(self):
        segment = self.segments[self._index]
        u, l = segment.upper_point, segment.lower_point

        self.Lp[l].add(segment)
        self.Up[u].add(segment)

        result = []
        u not in self.events and result.append(u)
        l not in self.events and result.append(l)

        # pridam body do seznamu udalosti
        map(lambda p: self.events.add(p), (u, l))

        if self._index+1 == len(self.segments):
            self.next_step = 'sweepline'
            self._index = 0
        else:
            self.next_step = 'events'

        self.step_info = "Počet událostí: " + str(len(self.events)) + '\n\n'

        self._index += 1
        return segment, result

    def do_step_sweepline(self):
        if len(self.events):
            self.next_step = 'sweepline-sets'
            self.event = self.events.pop(0)
            self.step_info = self.info_event()
            self.step_info += "Zbývá událostí: %d" % len(self.events)
            self.step_info += "\n\nOblast nad zametací přímkou a vlevo od bodu události má vyřešeny průsečíky."
            return self.event
        else:
            self.next_step = 'cycles'
            self.step_info = 'Všechny události byly zpracovány.\n\nNásleduje zpracování cyklů.'
            self._index = 0
            return None

    def do_step_sweepline_sets(self):
        self.next_step = 'sweepline-connect'
        self.step_info = self.info_event()

        self._lp = self.Lp[self.event]
        self._cp = self.Cp[self.event]
        self._up = self.Up[self.event]

        self.step_info += self.info_sets()

        self.step_info += self.info_tree()

        if len(self._cp) > 1:
            self.step_info += "\n" + 'Detekován nový průsečík!'
            self.next_step = 'sweepline-split'

        return self._cp

    def do_step_sweepline_split(self):
        self.next_step = 'sweepline-connect'
        self.step_info = self.info_event()

        update_segments = []

        while len(self._cp):
            # je v tom i odebrani usecky z C(p)
            s = self._cp.pop()

            # puvodni spodni bod uz neni aktualni (presouva se do p)
            self.Lp[s.lower_point].remove(s)

            # "rozsekani" usecky a hran
            new_s = self.split_cp_segment(s, self.event)
            self.Lp[new_s.lower_point].add(new_s)

            self._lp.add(s)
            self._up.add(new_s)

            update_segments.append(s)
            update_segments.append(new_s)

        self.step_info += self.info_sets()
        self.step_info += self.info_tree()
        self.step_info += '\nRozdělených hran: '+str(len(update_segments))+'\n'
        self.step_info += 'Nových hran: '+str(len(update_segments)/2)+'\n'

        return update_segments

    # spojeni hran
    def do_step_sweepline_connect(self):
        self.next_step = 'sweepline-findnew'
        self.step_info = self.info_event()
        self.step_info += self.info_sets()

        lup = self._lp.union(self._up)

        o1 = sum(map(lambda x: 1 if x.edge.source_map == 1 else 0, lup))
        o2 = sum(map(lambda x: 1 if x.edge.source_map == 2 else 0, lup))

        # ClockwiseConnectUpper
        if len(self._lp) > 0 and o1 > 0 and o2 > 0:
            lp_segments = filter(lambda s: s in self._lp, self.tree)
            sl = lp_segments[0]
            sr = lp_segments[-1]

            s = sl
            while s != sr:
                # s_r = pravy soused usecky s ve strome T
                s_r = self.tree[self.tree.bisect_right(s)]
                if s_r == s:
                    s_r = self.tree[self.tree.bisect_right(s)+1]
                self.clockwise_connect(s, s_r)
                s = s_r
        else:
            sl, sr = None, None

        # ----------------------------------------------------------------
        # Odeberu ze stromu T usecky L(p), pote do nej pridam usecky U(p)
        # ----------------------------------------------------------------
        if self._tree_event is None:
            self._tree_event = self.event

        # zrusim usecky L(p) ze stromu T (podle predchozi udalosti,
        # podle ktere se tam taky vkladaly)
        re_add = [s for s in self.tree if s not in self._lp]

        self._tree_event = self.event

        new_tree = sortedlist(key=self.tree_segment_order)
        map(lambda s: new_tree.add(s), re_add)

        search = self.SearchSegClass(a=self.event, b=self.event, name='search')
        left = new_tree.bisect_left(search)-1
        if left >= 0:
            self.left_segments[self.event] = new_tree[left]

        map(lambda s: new_tree.add(s), self._up)

        del self.tree
        self.tree = new_tree

        # ClockwiseConnectLower
        if len(self._up) > 0 and o1 > 0 and o2 > 0:
            up_segments = filter(lambda s: s in self._up, self.tree)
            tl = up_segments[0]
            tr = up_segments[-1]

            t = tr
            while t != tl:
                # s_r = pravy soused usecky s ve strome T
                t_l = self.tree[self.tree.bisect_left(t)-1]
                self.clockwise_connect(t, t_l)
                t = t_l
        else:
            tl, tr = None, None

        # oboji na propojeni
        if sl is not None and tl is not None:
            self.clockwise_connect(tl, sl)
            self.clockwise_connect(sr, tr)

        # IMHO nemuze nikdy nastat
        if sl is not None and tl is None:
            self.clockwise_connect(sr, sl)
        if sl is None and tl is not None:
            self.clockwise_connect(tl, tr)

        self.step_info += self.info_tree()
        self.step_info += "\nσ1(L(p) ∪ U(p)): {0}\nσ2(L(p) ∪ U(p)): {1}\n".format(o1, o2)

    def do_step_sweepline_findnew(self):
        self.step_info = self.info_event()
        self.step_info += self.info_sets()
        self.step_info += self.info_tree()

        new_events = []

        if len(self._up) == 0:
            search = self.SearchSegClass(a=self.event, b=self.event, name='search')
            i_l = self.tree.bisect_left(search)-1
            i_r = self.tree.bisect_right(search)
            sl = None if i_l < 0 else self.tree[i_l]
            sr = None if i_r > len(self.tree)-1 else self.tree[i_r]

            self.step_info += '\n' + "sl: {0}\nsr: {1}".format('není' if sl is None else sl.name,
                                                               'není' if sr is None else sr.name)
            new_events.append(self.find_new_event(sl, sr))
        else:
            self.step_info += '\n'

            s_ll = min(self._up, key=self.tree_segment_order)
            self.step_info += "s': " + s_ll.name + '\nsl: '
            i = self.tree.index(s_ll)-1

            if i >= 0:
                s_l = self.tree[i]
                self.step_info += s_l.name
                new_events.append(self.find_new_event(s_ll, s_l))
            else:
                self.step_info += 'není'

            self.step_info += '\n\n'

            s_rr = max(self._up, key=self.tree_segment_order)
            self.step_info += "s'': " + s_rr.name + '\nsr: '
            i = self.tree.index(s_rr)+1

            if len(self.tree) > i:
                s_r = self.tree[i]
                self.step_info += s_r.name
                new_events.append(self.find_new_event(s_rr, s_r))
            else:
                self.step_info += 'není'

        self.next_step = 'sweepline'

        return filter(lambda x: x is not None, new_events)

    def get_left_cycle(self, cycle):
        """
        Nalezne a vrati sousedni cyklus nalevo.
        """
        v = cycle.leftmost_vertex
        p = v.point

        # je to vnejsi neomezeny cyklus
        if p not in self.left_segments:
            return self.cycles[0]

        # usecka nalevo od cyklu, musime rozhodnout, kterou hranu
        segment = self.left_segments[p]
        edge = segment.edge if segment.edge.origin.point < p else segment.edge.twin
        return edge.cycle

    def do_step_cycles(self):
        self.next_step = 'components'
        self.step_info = 'Detekce cyklů\n\n'
        self.can_step = True

        c0 = Cycle(0, {(1, 'f0'), (2, 'f0')})
        self.cycles.append(c0)

        map(lambda e: setattr(e, 'is_processed', False), self.map.edges)

        i = 1

        for edge in self.map.edges:
            # bereme pouze nezpracovane hrany
            if edge.is_processed:
                continue
            # novy cyklu s indexem i
            c = Cycle(i)
            c.leftmost_edge = edge
            c.leftmost_vertex = edge.origin
            c.face_labels = set()

            l = edge.origin.point
            e = edge
            while True:
                setattr(e, 'cycle', c)
                v = e.origin.point
                if v.x < l.x or (v.x == l.x and v.y < l.y):
                    c.leftmost_vertex = e.origin
                    c.leftmost_edge = e
                    l = v
                c.face_labels.add((e.source_map, e.face_label))
                c.edges.append(e)
                e = e.next
                e.is_processed = True
                if e == edge:
                    break
            self.cycles.append(c)
            i += 1

        map(lambda e: delattr(e, 'is_processed'), self.map.edges)

        self.step_info += 'Bylo nalezeno {0} cyklů.'.format(len(self.cycles))
        self.step_info += '\n\nZařazení hrany do cyklu můžete nyní pozorovat v info oblasti.'

        return self.cycles

    def do_step_components(self):
        self.next_step = 'faces'
        self.step_info = ''
        self.can_step = True

        for cycle in self.cycles:
            # cyklus ma jiz komponentu prirazenu
            if cycle.component is not None:
                break

            if cycle.is_outer():
                g = Component()
                g.outer = cycle
                cycle.component = g
                self.components.append(g)
            else:
                cycles = [cycle]
                c = cycle
                while True:
                    c = self.get_left_cycle(c)
                    if c.component is None:
                        if c.is_outer():
                            g = Component()
                            g.outer = c
                            c.component = g
                            self.components.append(g)
                        else:
                            cycles.append(c)
                    # narazili jsme na cyklus, ktery ma komponentu
                    if c.component is not None:
                        break
                g = c.component
                # prihodime do komponenty vnitrni cykly
                for ic in cycles:
                    ic.component = g
                    g.inners.append(ic)

        self.step_info += "Počet nalezených komponent souvislosti: " + \
                          str(len(self.components))+'\n\n'
        self.step_info += self.info_components()

        return self.components

    def do_step_faces(self):
        self.next_step = 'finish'
        self.step_info += "Oblasti (komponenty):\n\n"
        self.step_info += self.info_components()

        for i, g in enumerate(self.components):
            for c in g.inners + [g.outer]:
                g.face_labels.update(c.face_labels)
                if len(g.face_labels) > 1:
                    break

            if len(g.face_labels) < 2:
                tmp = g
                while True:
                    c = tmp.outer
                    if c.index == 0:
                        missing = filter(lambda l: l[0] != list(g.face_labels)[0][0],
                                         c.face_labels)
                        g.face_labels.update(missing)
                        break
                    e = c.leftmost_edge.twin
                    tmp = e.cycle.component
                    outer_cycle = tmp.outer
                    missing = filter(lambda l: l[0] != list(g.face_labels)[0][0],
                                     outer_cycle.face_labels)
                    g.face_labels.update(missing)

                    if len(g.face_labels) > 1:
                        break

        for g in self.components:
            face = Face('', outer_edge=g.outer.leftmost_edge)
            face.inner_edges = map(lambda c: c.leftmost_edge, g.inners)

            # hrany pro nastaveni nove oblasti
            edges = []
            edges.extend(g.outer.edges)
            map(lambda x: edges.extend(x.edges), g.inners)

            # nastavim nove oblasti hranam
            map(lambda e: setattr(e, 'face', face), edges)

            labels = list(g.face_labels)
            face.name = "+".join(map(lambda l: str(l[0])+'_'+l[1], labels))

            self.map.faces.append(face)

    def do_step_finish(self):
        self.can_step = False
        self.step_info = 'Překryv map byl dokončen!\n\n' \
                         'Můžete si jej uložit: \nMenu -> Soubor -> Uložit překryv'
        self.step_info += '\n\nNázvy hran a vrcholů budou při uložení přejmenovány!'
        self.step_info += '\n\nNázvy původních oblastí zůstanou zachovány s prefixem čísla mapy:\n1_, 2_'


def vertical_separator(parent, row=0, column=0, rowspan=1, columnspan=1, **kwargs):
    """
    Vertikální oddělovač v GUI.
    """
    sep = Frame(parent, relief=RIDGE, width=2, bd=2, **kwargs)
    sep.grid(row=row, column=column, rowspan=rowspan, columnspan=columnspan, sticky=NSEW, padx=1)


def horizontal_separator(parent, row=0, column=0, rowspan=1, columnspan=1, **kwargs):
    """
    Horizontální oddělovač v GUI.
    """
    sep = Frame(parent, relief=RIDGE, height=2, bd=2, **kwargs)
    sep.grid(row=row, column=column, rowspan=rowspan, columnspan=columnspan, sticky=NSEW, pady=1)


class MapOverlayGUI(object):
    FONT_STEP = "Font 13"
    FONT_SUB_STEP = "Font 12"

    MAP_COLORS = ['white', 'yellow']

    def __init__(self, root):
        """
         :param root: Hlavní (mateřské okno)
         :type root: Tk
        """
        self._new_segment = None

        self.root = root
        root.title("MapOverlay")
        root.minsize(950, 650)
        root.geometry("950x650+300+100")

        menu = Menu(root)

        # menu soubor
        filemenu = Menu(menu, tearoff=0)
        filemenu.add_command(label="Vybrat 1. mapu...", command=self.choose_map1)
        filemenu.add_command(label="Vybrat 2. mapu...", command=self.choose_map2)
        filemenu.add_separator()
        filemenu.add_command(label="Uložit překryv...", command=self.save_map)
        filemenu.add_separator()
        filemenu.add_command(label="Ukončit program", command=root.quit)
        menu.add_cascade(label="Soubor", menu=filemenu)

        root.config(menu=menu)

        # odkazy na menu budu potrebovat, tak si je ulozim...
        self.menu = menu
        self.menu_file = filemenu

        # sestveni GUI okna
        self.canvas = Canvas(self.root, background='black', bd=0, cursor='crosshair')
        self.canvas.grid(row=0, column=0, sticky=NSEW, columnspan=2, pady=0, padx=0)

        self.canvas.bind('<Motion>', self.on_canvas_motion)
        self.canvas.bind('<Enter>', self.on_canvas_enter)
        self.canvas.bind('<Leave>', self.on_canvas_leave)
        self.canvas.bind('<B1-Motion>', self.on_canvas_btn1_motion)

        # popisky jednotlivych objektu na platne (muzou byt prazdne)
        self.canvas_labels = dict()

        # uschova nejblizsi polozky (pro restore konfigurace)
        self._item_closest = None
        self._item_closest_config = None
        # Je posleni polozka usecka? (kvuli zobrazovani sipek u hran)
        self._item_closest_segment = None

        self._pushed_configurations = defaultdict(dict)

        # punticek udalosti ke smazani
        self._event_to_delete = None

        # Algoritmus MapOverlay
        self.alg = MapOverlayAlgorithm()

        # info o ukazateli mysi ([34;32])
        self.status_pos = Label(self.root, text="", width=20, bg='black', fg='white', font="Font 11")
        self.status_pos.grid(row=1, column=0, padx=3)

        self.status_item = Label(self.root, text="", bg='black', fg='white', anchor=W, padx=4,
                                 font='Font 11')
        self.status_item.grid(row=1, column=1, sticky=EW, padx=3)

        # vertikalni separator oddelujici canvas a info panel
        vertical_separator(self.root, row=0, column=2, rowspan=2)

        # --------------------- INFO -------------------------
        info = Frame(self.root, relief=FLAT, width=230, padx=4, pady=4)
        info.grid(column=3, row=0, rowspan=2, sticky=NSEW)

        l1 = Label(info, text="Kroky algoritmu:", anchor=W)
        l1.grid(row=0, column=0, sticky=EW)

        horizontal_separator(info, row=1, column=0)

        # ------------ INFO, FAZE ALGORITMU -------------------
        wsteps = Frame(info, pady=10)

        self.labels_steps = []

        for step in range(len(self.alg.steps)):
            code = self.alg.steps[step][0]
            text = self.alg.steps[step][1]
            if '-' in code:
                text = ' - ' + text
            label = Label(wsteps, text=text, anchor=W, padx=5)
            label.grid(row=step, sticky=W)
            # naformatovani fontu a barvy probiha ve funkci self.update_steps
            self.labels_steps.append(label)

        wsteps.grid(row=2, sticky=EW)

        horizontal_separator(info, row=3, pady=10)

        # ------------ INFO ovladaci prvky
        pw = PanedWindow(info, orient=HORIZONTAL)
        pw.grid(row=4, sticky=EW)

        self.btn_step = Button(pw, text="Dále", command=self.on_step_click, width=10)
        pw.add(self.btn_step, sticky=EW)

        self.btn_next = Button(info, text="Vše", command=self.on_all_click)
        pw.add(self.btn_next, sticky=EW)

        horizontal_separator(info, row=5)

        # ------------ INFO zobrazovani info k aktualnimu kroku
        self.step_info = Label(info, text='', anchor=W, justify=LEFT, wraplength=220, font="Font 12")
        self.step_info.grid(row=10, sticky=NSEW, pady=5)


        # ----------- Tlacitko na ukonceni aplikace
        pw2 = PanedWindow(info, orient=VERTICAL)
        pw2.add(Frame(pw2, relief=RIDGE, height=2, pady=4, bd=2))
        # Tlacitko na ukoneceni aplikace
        self.btn_quit = Button(info, text="Ukončit program", fg='red', command=root.quit)
        pw2.add(self.btn_quit)
        pw2.grid(row=20, columnspan=2, sticky=S+E+W)

        # ------------ Nakonfigurovani INFO panelu
        info.grid_rowconfigure(20, weight=1)
        info.grid_columnconfigure(0, weight=1)

        # nastavim rozpinani 0;0 - canvas
        self.root.grid_columnconfigure(1, minsize=230, weight=1)
        self.root.grid_rowconfigure(0, weight=1)

        # rozpinani info panelu
        self.root.grid_columnconfigure(3, minsize=230, weight=0)

        self.alg.canvas = self.canvas

        # zacneme nultou operaci (simulujeme kliknuti na tlacitko Dalsi)
        self.on_step_click()

        # vyzdvihnuti okna do popredi
        root.lift()
        root.focus_set()
        self.root.call('wm', 'attributes', '.', '-topmost', True)
        self.root.after_idle(self.root.call, 'wm', 'attributes', '.', '-topmost', False)

    def choose_map1(self):
        self.dlg_choose_map(1)

    def choose_map2(self):
        self.dlg_choose_map(2)

    def dlg_choose_map(self, index):
        """
        Zobrazí dialog pro výběr první/druhé mapy.
        :param index: Je-li 1, vybírá se první mapa, jinak druhá.
        """
        filename = tkFileDialog.askopenfilename(
            title="Vybrat " + ("první" if index == 1 else "druhou") + " mapu",
            filetypes=(('XML mapy', '*.xml'),),
            parent=self.root,
            initialdir=os.getcwd()
        )

        if filename is None:
            return

        alg_map = self.alg.map1 if index == 1 else self.alg.map2
        alg_map.clear()
        alg_map.load_from_file(filename)

        # byly nacteny obe mapy, povolim tedy dalsi krok
        if len(self.alg.map1.edges) > 0 and len(self.alg.map2.edges) > 0:
            self.btn_step['state'] = NORMAL

        # aktualizujeme gui (prekreslime mapu)
        self.update_step_gui(self.alg.step, alg_map)

    def save_map(self):
        """
        Ulozi DCEL stavajici mapy do zvoleneho souboru ve formatu XML.
        """
        xmlfile = tkFileDialog.asksaveasfile(
            mode='w',
            title="Uložení překryvu",
            parent=self.root,
            initialdir=os.getcwd()
        )

        if xmlfile is not None:
            # prejmenujeme na unikatni nazvy
            for i, v in enumerate(self.alg.map.vertices):
                v.name = 'v'+str(i)
            for i, e in enumerate(self.alg.map.edges):
                e.name = 'e'+str(i)

            xmlfile.write(self.alg.map.xml)
            xmlfile.close()

    def update_steps(self):
        codes = self.alg.step_codes
        step_index = codes.index(self.alg.step)

        last_big_step = None

        for index, label in enumerate(self.labels_steps):
            # Velky krok, obsahujici nekolik podkroku
            if not '-' in codes[index]:
                last_big_step = codes[index]
                font = self.FONT_STEP
            else:
                font = self.FONT_SUB_STEP

            label['font'] = font

            if index < step_index and not self.alg.step.startswith(last_big_step):
                label['fg'] = 'dark green'
            elif index == step_index:
                label['fg'] = 'black'
                label['font'] += ' bold'
            else:
                label['fg'] = '#777'

    def update_step_gui(self, step, result):
        """
        Aktualizace (reakce) GUI na provedeny krok.
        """
        self.step_info['text'] = self.alg.step_info

        if step == 'input':
            if result is None:
                return

            # byla vybrana mapa (result neni None), takze ji zobrazime
            map_index = '1' if result == self.alg.map1 else '2'

            # smazeme objekty puvodni mapy, prepisujeme na novou
            self.canvas.delete('map'+map_index)

            for s in result.segments:
                self.canvas.create_line(s.a.x, s.a.y, s.b.x, s.b.y,
                                        fill=self.MAP_COLORS[int(map_index)-1],
                                        tags=('segment', 'map'+map_index))

        if step == 'init':
            self.menu_file.entryconfig(0, state=DISABLED)
            self.menu_file.entryconfig(1, state=DISABLED)

            # vymazeme stare (preview)
            self.canvas.delete('segment')

            for i, seg in enumerate(self.alg.segments):
                self.canvas_create_segment(seg, i)

        # inicializace algoritmu
        if step == 'events':
            segment = result[0]

            self.canvas_item_config_restore_all()

            # ulozime nastaveni teto hrany
            self.canvas_item_config_save(segment.canvas_item)
            self.canvas.itemconfig(segment.canvas_item, fill='red')

            for p in result[1]:
                item = self.canvas.create_oval(p.x-3, p.y-3, p.x+3, p.y+3,
                                               fill='red', outline='white', tags='event')
                # ulozim si cislo prvku do bodu udalosti, at to pak muzu dohledat
                p.tag = item
                self.canvas_labels[item] = 'Bod události p=[{0};{1}]'.format(int(p.x), int(p.y))

        if step == 'sweepline':
            # z predchoziho kroku
            if self.alg.prev_step == 'events':
                self.canvas_item_config_restore_all()

            # ten predchozi
            self.canvas_delete_event()

            # je to udalost?
            if result is not None:
                event = result
                # vytvorim objekt zametaci primky
                self.canvas_create_sweepline(event.y)
                self.canvas.itemconfig(event.tag, fill='green')
                self._event_to_delete = event.tag
            else:
                self.canvas_delete_sweepline()

        if step == 'event-sets':
            event = self.alg.event
            # pokud to je prusecik, prebarvim ho na bilou, pokud ne, necham to tak,
            # ale zaridim jeho odstraneni. Result vraci L(p) u C(p) u U(p)
            if len(result) > 1:
                self.canvas.itemconfig(event.tag, fill='white')
                #self.canvas_labels[event.tag] = "Průsečík úseček " + ", ".join([s.name for s in result])
            else:
                self._event_to_delete = event.tag

        if step == 'sweepline-split':
            # ve vysledku jsou pozmenene usecky
            update_segments = result

            index = len(self.alg.segments) - len(update_segments)/2

            for segment in update_segments:

                # jiz existujici usecka na platne -> upravim souradnice
                if hasattr(segment, 'canvas_item'):
                    x1, y1 = int(segment.a.x), int(segment.a.y)
                    x2, y2 = int(segment.b.x), int(segment.b.y)
                    self.canvas.coords(segment.canvas_item, x1, y1, x2, y2)
                else:
                    #i = self.alg.segments.index(segment, -(len(update_segments)/2))
                    self.canvas_create_segment(segment, index)
                    index += 1

        if step == 'sweepline-connect':
            pass

        if step == 'sweepline-findnew':
            for p in result:
                item = self.canvas.create_oval(float(p.x)-3, float(p.y)-3, float(p.x)+3, float(p.y)+3,
                                               fill='yellow', outline='white', tags='event')
                # ulozim si cislo prvku do bodu udalosti, at to pak muzu dohledat
                p.tag = item
                self.canvas_labels[item] = 'Bod události p=[{0};{1}]'.format(int(p.x), int(p.y))
            #self.canvas.delete(event.tag)

        if step == 'cycles':
            cycles = result

            for cycle in cycles:
                if cycle.leftmost_vertex is None:
                    continue
                p = cycle.leftmost_vertex.point
                oval = self.canvas.create_oval(float(p.x)-2, float(p.y)-2, float(p.x)+2, float(p.y)+2,
                                               fill='grey', outline='white', tags='event')
                self.canvas_labels[oval] = "Nejlevější bod cyklu c"+str(cycle.index)


        if step == 'components':
            components = result

    def canvas_item_config_save(self, item):
        """
        Ulozi nastaveni objektu na platne.
        """
        config = self.canvas.itemconfigure(item)
        for key in config.keys():
            self._pushed_configurations[item][key] = config[key][4]

    def canvas_item_config_restore(self, item):
        """
        Obnovi ulozene nastaveni objektu platna (pokud nejake je).
        """
        if item in self._pushed_configurations:
            self.canvas.itemconfig(item, **self._pushed_configurations.pop(item))

    def canvas_item_config_restore_all(self):
        """
        Obnovi nastaveni vsech objektu, ktera jsou ulozena.
        """
        while len(self._pushed_configurations):
            item, config = self._pushed_configurations.popitem()
            self.canvas.itemconfig(item, **config)

    def canvas_create_sweepline(self, y):
        """
        Vytvori na platne zametaci primku.
        """
        w = self.canvas.winfo_width()
        items = self.canvas.find_withtag('sweepline')
        y = float(y)
        # aktualizuji data u stavajici
        if len(items):
            l = items[0]
            self.canvas.coords(l, 0, y, w, y)
        else:
            l = self.canvas.create_line(0, y, w, y, fill='white', dash=(3, 4), tags='sweepline')
            self.canvas_labels[l] = "Zametací přímka"

    def canvas_delete_sweepline(self):
        """
        Odstrani zametaci primku z platna.
        """
        # nemusi byt zadana ani jedna usecka a tohle muze byt prazdne
        items = self.canvas.find_withtag('sweepline')
        if len(items):
            sweepline = items[0]
            self.canvas.delete(sweepline)
            del self.canvas_labels[sweepline]

    def canvas_delete_event(self):
        if self._event_to_delete is not None:
            self.canvas.delete(self._event_to_delete)
            self._event_to_delete = None

    def canvas_create_segment(self, segment, index):
        """
        Vytvori usecku na platne, ktera patri nejake mape.
        :param segment: Usecka k vytvoreni.
        :param index: Poradi usecky v seznamu usecek alg.segments.
        :return: item id
        """
        tags = ('segment', 'map'+str(segment.edge.source_map), 'i'+str(index))
        item = self.canvas.create_line(int(segment.a.x), int(segment.a.y),
                                       int(segment.b.x), int(segment.b.y),
                                       fill=self.MAP_COLORS[segment.edge.source_map-1],
                                       tags=tags)
        # nastavim usecce cislo objektu v canvasu (budu pouzivat)
        setattr(segment, 'canvas_item', item)
        return item

    def on_canvas_btn1_motion(self, event):
        """
        Obsluha udalosti tahnuti mysi se stisknutym tlacitkem.
        """
        self.status_pos['text'] = "X={0}  Y={1}".format(event.x, event.y)

    def on_canvas_motion(self, event):
        """
        Obsluha udalosti pohybu kurzoru mysi.
        :param event: Udalost
        """
        self.status_pos['text'] = "X={0}  Y={1}".format(event.x, event.y)

        items = self.canvas.find_closest(event.x, event.y, 1)

        # existuje nejblizi objekt ke kurzoru mysi a je jiny nez ten predchozi!
        if len(items) and items[0] != self._item_closest:
            item = items[0]
            # predchozi polozka neni prazdna -> je treba obnovit puvodni konfiguraci
            if self._item_closest is not None:
                self.canvas.itemconfig(self._item_closest, **self._item_closest_config)
            self._item_closest = item
            self._item_closest_config = {'width': self.canvas.itemcget(item, 'width')}

            self.canvas.itemconfig(item, width=2)

            tags = self.canvas.itemcget(item, 'tags').split(' ')
            if 'segment' in tags and self.alg.step != 'input':
                self._item_closest_config['arrow'] = ''
                self._item_closest_segment = int(filter(lambda x: x.startswith('i'), tags)[0][1:])
            else:
                self._item_closest_segment = None

            # upgradujeme status objektu
            self.status_item['text'] = self.canvas_labels[item] if item in self.canvas_labels else ''

        # nejblizsi objekt je hranou, zobrazim info
        if self._item_closest_segment is not None:
            item = items[0]
            index = self._item_closest_segment
            segment = self.alg.segments[index]

            # info o stranach
            sides = ((LAST, segment.a, segment.a.distance(event)),
                     (FIRST, segment.b, segment.b.distance(event)))

            # na ktere strane usecky ma byt sipka
            side = min(sides, key=lambda x: x[2])

            self.canvas.itemconfig(item, arrow=side[0])

            # ktera hrana je blize kurzoru? tu zobrazime v popisku
            edge = segment.edge if segment.edge.origin.point == side[1] else segment.edge.twin

            self.status_item['text'] = 'Úsečka s{0}, hrana {1.name}: twin={1.twin.name}, ' \
                                       'next={1.next.name}, prev={1.prev.name}, face={1.face_label}'.format(index, edge)

            # hrana ma nastaveny cyklus
            if hasattr(edge, 'cycle'):
                cycle = edge.cycle
                border = 'vnitřní' if cycle.is_inner() else 'vnější'
                self.status_item['text'] += ' | ' + border + ' cyklus c' + str(cycle.index)

                # cyklus ma prirazenu komponentu
                if hasattr(cycle, 'component'):
                    g = cycle.component
                    if g is not None:
                        self.status_item['text'] += ' | komponenta g'+str(self.alg.components.index(g))

    def on_canvas_leave(self, event):
        self.status_pos['text'] = ''
        self.status_item['text'] = ''

        if self._item_closest is not None:
            self.canvas.itemconfig(self._item_closest, **self._item_closest_config)
            self._item_closest = None
            self._item_closest_config = None

    def on_canvas_enter(self, event):
        self.status_pos['text'] = "X={0}  Y={1}".format(event.x, event.y)

    def on_all_click(self):
        codes = self.alg.step_codes
        curr_index = codes.index(self.alg.step)
        # budeme kroky vykonavat do te doby, nez stavajici predstihne nasledujici
        while True:
            self.on_step_click()
            if codes.index(self.alg.step) > curr_index and '-' not in self.alg.step:
                break

    def on_step_click(self):
        result = self.alg.do_step()
        self.update_steps()
        self.btn_step['state'] = NORMAL if self.alg.can_step else DISABLED
        self.btn_next['state'] = NORMAL if self.alg.can_step else DISABLED
        self.update_step_gui(self.alg.step, result)
        self.root.update_idletasks()


# Inicializace Tk a GUI
tk = Tk()
win = MapOverlayGUI(tk)
mainloop()
